#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_file.interfaces import IImage, IThumnailImageWidget, IThumbnail
from pyams_form.interfaces.form import IWidgetsPrefixViewletsManager, IFormHelp
from pyams_skin.interfaces.viewlet import IContextActions
from pyams_skin.layer import IPyAMSLayer
from pyams_utils.interfaces import MANAGE_PERMISSION
from pyams_zmi.layer import IAdminLayer

# import packages
from pyams_file.image import ThumbnailGeometrry
from pyams_form.form import AJAXEditForm
from pyams_form.help import FormHelp
from pyams_form.schema import CloseButton
from pyams_pagelet.pagelet import pagelet_config
from pyams_skin.viewlet.toolbar import ToolbarMenuItem, ToolbarMenuDivider
from pyams_template.template import template_config
from pyams_utils.adapter import adapter_config
from pyams_viewlet.viewlet import viewlet_config, Viewlet
from pyams_zmi.form import AdminDialogEditForm
from pyramid.view import view_config
from z3c.form import field, button
from zope.interface import implementer, Interface
from zope.schema import Int, Bool
from zope.schema.fieldproperty import FieldProperty

from pyams_file import _


@viewlet_config(name='image.resize.divider', context=IImage, layer=IPyAMSLayer, view=Interface,
                manager=IContextActions, permission=MANAGE_PERMISSION, weight=19)
class ImageDividerAction(ToolbarMenuDivider):
    """Image divider action"""


#
# Image resize
#

@viewlet_config(name='image.resize.action', context=IImage, layer=IPyAMSLayer, view=Interface,
                manager=IContextActions, permission=MANAGE_PERMISSION, weight=20)
class ImageResizeAction(ToolbarMenuItem):
    """Image resize action"""

    label = _("Resize image...")
    label_css_class = 'fa fa-fw fa-compress'

    url = 'resize.html'
    modal_target = True


class IResizeFormButtons(Interface):
    """Image resize form buttons"""

    close = CloseButton(name='close', title=_("Close"))
    resize = button.Button(name='resize', title=_("Resize image"))


class IImageResizeInfo(Interface):
    """Image resize interface"""

    width = Int(title=_("New image width"))

    height = Int(title=_("New image height"))

    keep_ratio = Bool(title=_("Keep aspect ratio"),
                      description=_("Check to keep original aspect ratio; image will be resized as large as "
                                    "possible within given limits"),
                      required=True,
                      default=True)


@implementer(IImageResizeInfo)
class ImageResizeInfo(object):
    """Image resize info"""

    width = FieldProperty(IImageResizeInfo['width'])
    height = FieldProperty(IImageResizeInfo['height'])
    keep_ratio = FieldProperty(IImageResizeInfo['keep_ratio'])


@adapter_config(context=IImage, provides=IImageResizeInfo)
def ImageResizeInfoFactory(image):
    size = image.get_image_size()
    info = ImageResizeInfo()
    info.width = size[0]
    info.height = size[1]
    return info


@pagelet_config(name='resize.html', context=IImage, layer=IPyAMSLayer, permission=MANAGE_PERMISSION)
class ImageResizeForm(AdminDialogEditForm):
    """Image resize form"""

    legend = _("Resize image")
    icon_css_class = 'fa fa-fw fa-compress'

    fields = field.Fields(IImageResizeInfo)
    buttons = button.Buttons(IResizeFormButtons)
    ajax_handler = 'resize.json'

    @property
    def title(self):
        return self.context.title or self.context.filename

    def updateActions(self):
        super(ImageResizeForm, self).updateActions()
        if 'resize' in self.actions:
            self.actions['resize'].addClass('btn-primary')


@view_config(name='resize.json', context=IImage, request_type=IPyAMSLayer,
             permission=MANAGE_PERMISSION, renderer='json', xhr=True)
class ImageResizeAJAXForm(AJAXEditForm, ImageResizeForm):
    """Image resize form, AJAX renderer"""

    def update_content(self, content, data):
        self.context.resize(data.get('width'), data.get('height'), data.get('keep_ratio'))

    def get_ajax_output(self, changes):
        return {'status': 'reload',
                'smallbox': self.request.localizer.translate(self.successMessage),
                'smallbox_status': 'success'}


@adapter_config(context=(IImage, IAdminLayer, ImageResizeForm), provides=IFormHelp)
class ImageResizeFormHelpAdapter(FormHelp):
    """Image resize form help adapter"""

    message = _("""You can use this form to change image dimensions.

This will generate a new image only if requested size is smaller than the original one.""")
    message_format = 'rest'


#
# Image crop
#

@viewlet_config(name='image.crop.action', context=IImage, layer=IPyAMSLayer, view=Interface,
                manager=IContextActions, permission=MANAGE_PERMISSION, weight=21)
class ImageCropAction(ToolbarMenuItem):
    """Image crop action"""

    label = _("Crop image...")
    label_css_class = 'fa fa-fw fa-crop'

    url = 'crop.html'
    modal_target = True


class ICropFormButtons(Interface):
    """Image crop form buttons"""

    close = CloseButton(name='close', title=_("Close"))
    crop = button.Button(name='crop', title=_("Crop"))


@pagelet_config(name='crop.html', context=IImage, layer=IPyAMSLayer, permission=MANAGE_PERMISSION)
class ImageCropForm(AdminDialogEditForm):
    """Image crop form"""

    legend = _("Crop image")
    icon_css_class = 'fa fa-fw fa-crop'
    dialog_class = 'modal-large'

    fields = field.Fields(Interface)
    buttons = button.Buttons(ICropFormButtons)
    ajax_handler = 'crop.json'

    @property
    def title(self):
        return self.context.title or self.context.filename

    def updateActions(self):
        super(ImageCropForm, self).updateActions()
        if 'crop' in self.actions:
            self.actions['crop'].addClass('btn-primary')


@view_config(name='crop.json', context=IImage, request_type=IPyAMSLayer,
             permission=MANAGE_PERMISSION, renderer='json', xhr=True)
class ImageCropAJAXForm(AJAXEditForm, ImageCropForm):
    """Image crop form, AJAX renderer"""

    def update_content(self, content, data):
        image_size = self.context.get_image_size()
        x1 = int(self.request.params.get('selection.x1', 0))
        y1 = int(self.request.params.get('selection.y1', 0))
        x2 = int(self.request.params.get('selection.x2', image_size[0]))
        y2 = int(self.request.params.get('selection.y2', image_size[1]))
        self.context.crop(x1, y1, x2, y2)

    def get_ajax_output(self, changes):
        return {'status': 'reload',
                'smallbox': self.request.localizer.translate(self.successMessage),
                'smallbox_status': 'success'}


@viewlet_config(name='crop.widgets-prefix', context=IImage, layer=IAdminLayer, view=ImageCropForm,
                manager=IWidgetsPrefixViewletsManager)
@template_config(template='templates/image-crop.pt')
class ImageCropViewletsPrefix(Viewlet):
    """Image crop viewlets prefix"""


#
# Image square thumbnail selection
#

@viewlet_config(name='image.thumb.divider', context=IImage, layer=IPyAMSLayer, view=IThumnailImageWidget,
                manager=IContextActions, permission=MANAGE_PERMISSION, weight=30)
class ImageThumbnailsDividerAction(ToolbarMenuDivider):
    """Image divider action"""


class IThumbnailFormButtons(Interface):
    """Image crop form buttons"""

    close = CloseButton(name='close', title=_("Close"))
    crop = button.Button(name='crop', title=_("Select thumbnail"))


@viewlet_config(name='image.thumb.square.action', context=IImage, layer=IPyAMSLayer, view=IThumnailImageWidget,
                manager=IContextActions, permission=MANAGE_PERMISSION, weight=31)
class ImageSquareThumbnailAction(ToolbarMenuItem):
    """Square thumbnail image selection"""

    label = _("Select square thumbnail...")
    label_css_class = 'fa fa-fw fa-instagram'

    url = 'square-thumbnail.html'
    modal_target = True


@pagelet_config(name='square-thumbnail.html', context=IImage, layer=IPyAMSLayer, permission=MANAGE_PERMISSION)
class ImageSquareThumbnailEditForm(AdminDialogEditForm):
    """Image square thumbnail edit form"""

    legend = _("Select square thumbnail")
    icon_css_class = 'fa fa-fw fa-instagram'
    dialog_class = 'modal-large'

    fields = field.Fields(Interface)
    buttons = button.Buttons(IThumbnailFormButtons)
    ajax_handler = 'square-thumbnail.json'

    @property
    def title(self):
        return self.context.title or self.context.filename

    def updateActions(self):
        super(ImageSquareThumbnailEditForm, self).updateActions()
        if 'crop' in self.actions:
            self.actions['crop'].addClass('btn-primary')


@view_config(name='square-thumbnail.json', context=IImage, request_type=IPyAMSLayer,
             permission=MANAGE_PERMISSION, renderer='json', xhr=True)
class ImageSquareThumbnailAJAXEditForm(AJAXEditForm, ImageSquareThumbnailEditForm):
    """Image square thumbnail edit form, AJAX renderer"""

    def update_content(self, content, data):
        geometry = ThumbnailGeometrry()
        geometry.x1 = int(self.request.params.get('selection.x1'))
        geometry.y1 = int(self.request.params.get('selection.y1'))
        geometry.x2 = int(self.request.params.get('selection.x2'))
        geometry.y2 = int(self.request.params.get('selection.y2'))
        IThumbnail(self.context).set_thumbnail_geometry('square', geometry)

    def get_ajax_output(self, changes):
        return {'status': 'success',
                'smallbox': self.request.localizer.translate(self.successMessage),
                'smallbox_status': 'success'}


@viewlet_config(name='square-thumbnail.widgets-prefix', context=IImage, layer=IAdminLayer,
                view=ImageSquareThumbnailEditForm, manager=IWidgetsPrefixViewletsManager)
@template_config(template='templates/image-square-thumbnail.pt')
class ImageSquareThumbnailViewletsPrefix(Viewlet):
    """Image square thumbnail viewlets prefix"""


#
# Image panoramic thumbnail selection
#

@viewlet_config(name='image.thumb.pano.action', context=IImage, layer=IAdminLayer, view=IThumnailImageWidget,
                manager=IContextActions, permission=MANAGE_PERMISSION, weight=32)
class ImagePanoThumbnailAction(ToolbarMenuItem):
    """Panoramic thumbnail image selection"""

    label = _("Select panoramic thumbnail...")
    label_css_class = 'fa fa-fw fa-youtube-play'

    url = 'pano-thumbnail.html'
    modal_target = True

    def updateActions(self):
        super(ImageSquareThumbnailEditForm, self).updateActions()
        if 'crop' in self.actions:
            self.actions['crop'].addClass('btn-primary')


@pagelet_config(name='pano-thumbnail.html', context=IImage, layer=IPyAMSLayer, permission=MANAGE_PERMISSION)
class ImagePanoThumbnailEditForm(AdminDialogEditForm):
    """Image panoramic thumbnail edit form"""

    legend = _("Select panoramic thumbnail")
    icon_css_class = 'fa fa-fw fa-youtube-play'
    dialog_class = 'modal-large'

    fields = field.Fields(Interface)
    buttons = button.Buttons(IThumbnailFormButtons)
    ajax_handler = 'pano-thumbnail.json'

    @property
    def title(self):
        return self.context.title or self.context.filename

    def updateActions(self):
        super(ImagePanoThumbnailEditForm, self).updateActions()
        if 'crop' in self.actions:
            self.actions['crop'].addClass('btn-primary')


@view_config(name='pano-thumbnail.json', context=IImage, request_type=IPyAMSLayer,
             permission=MANAGE_PERMISSION, renderer='json', xhr=True)
class ImagePanoThumbnailAJAXEditForm(AJAXEditForm, ImagePanoThumbnailEditForm):
    """Image panoramic thumbnail edit form, AJAX renderer"""

    def update_content(self, content, data):
        geometry = ThumbnailGeometrry()
        geometry.x1 = int(self.request.params.get('selection.x1'))
        geometry.y1 = int(self.request.params.get('selection.y1'))
        geometry.x2 = int(self.request.params.get('selection.x2'))
        geometry.y2 = int(self.request.params.get('selection.y2'))
        IThumbnail(self.context).set_thumbnail_geometry('pano', geometry)

    def get_ajax_output(self, changes):
        return {'status': 'success',
                'smallbox': self.request.localizer.translate(self.successMessage),
                'smallbox_status': 'success'}


@viewlet_config(name='pano-thumbnail.widgets-prefix', context=IImage, layer=IAdminLayer,
                view=ImagePanoThumbnailEditForm, manager=IWidgetsPrefixViewletsManager)
@template_config(template='templates/image-pano-thumbnail.pt')
class ImagePanoThumbnailViewletsPrefix(Viewlet):
    """Image panoramic thumbnail viewlets prefix"""
